/* ============================================================ *
 * reduced_fit.demo.c						*
 * Martin Kilbinger (2010)					*
 *								*
 * Demo program for the reduced-shear correction fits, see K10.	*
 * You have to fill in the calculation of \bar F^{mn}(a), which *
 * involves the comoving distance and galaxy redshift		*
 * distribution. See K10 eqs (10,12).				*
 * ============================================================ */

#include "reduced_fit.h"

#define EPS 1.0e-6

/* Returns \bar F^{mn}(a), K10 (10, 12) */
double Fbar_mn(double a, const double *p0)
{
   // \bar F^{mn} = 0.5(dchi/da * G^m(a) G^n(a) * (G^n(a) + G^m(a)));

   return 0.0;
}

/* Numerical derivative d[\bar F^{mn}(a)]/dp_alpha */
double dFbar_mn_dp_alpha(double a, int alpha, const double *p0)
{
   double p0_fid[M_PAR], h, fp, fm;
   int m;

   /* Copy fiducial parameter */
   for (m=0; m<M_PAR; m++) p0_fid[m] = p0[m];
 
   /* Finite difference = fraction of fiducial parameter */
   h = p0[alpha]*FH;

   p0_fid[alpha] = p0[alpha] + h;
   fp = Fbar_mn(a, p0_fid);

   p0_fid[alpha] = p0[alpha] - h;
   fm = Fbar_mn(a, p0_fid);
   
   return (fp-fm)/(2.0*h);
}

int main(int argc, const char *argv[])
{
   /* Fiducial cosmology (WMAP7): (unused Omegam Omegade w0 Omegab h100 sigma8 nspec) */
   const double p0[M_PAR] = {0.0, 0.27, 0.73, -1.0, 0.045, 0.71, 0.8, 0.96};

   double p[M_PAR], dpar[M_PAR];
   int alpha, k, Na;
   double a, a_min, da, ell, Pg1, *fmn, *dfmn_dp[M_PAR];


   /* Example cosmologogy */
   p[0] = 0.0;   /* Unused */
   p[1] = 0.25;  /* Omegam */
   p[2] = 0.75;  /* Omegade */
   p[3] = -1.0;  /* w0 */
   p[4] = 0.045; /* Omegab */
   p[5] = 0.71;  /* h100 */
   p[6] = 0.85;  /* sigma8 */
   p[7] = 0.96;  /* ns */


  /* Calculate dpar = p - p_0 */
   for (alpha=1; alpha<M_PAR; alpha++) {
      dpar[alpha] = p[alpha] - p0[alpha];
   }

   if (check_limits(dpar)==1) {
      fprintf(stderr, "Error: Parameter outside limits\n");
      return 1;
   }


   /* Scale-factor grid */
   a_min = 1.0/(1.0+2.0);   /* z=2 */
   Na    = 20;
   da    = (1.0-a_min)/(double)Na;


   /* Allocate memory */
   fmn = calloc(Na, sizeof(double));
   for (alpha=1; alpha<M_PAR; alpha++) {
      dfmn_dp[alpha] = calloc(Na, sizeof(double));
   }


   /* Calculate F^{mn}, K10 (10,12) at fiducial cosmology */
   fprintf(stderr, "Fill in the function Fbar_mn\n");
   for (k=0,a=a_min; k<Na; k++,a+=da) {
      fmn[k] = Fbar_mn(a, p0);
   }

   /* Calculate dF^{mn}/dp at fiducial cosmology */
   for (alpha=1; alpha<M_PAR; alpha++) {
      if (fabs(dpar[alpha])>EPS) {
	 for (k=0,a=a_min; k<Na; k++,a+=da) {
	    dfmn_dp[alpha][k] = dFbar_mn_dp_alpha(a, alpha, p0);
	 }
      }
   }


   /* Calculate reduced-shear correction */
   ell = 3000.0;
   Pg1 = sum_a_for_Pg1(log(ell), a_min, Na, da, fmn, (const double**)dfmn_dp, dpar);

   //printf("Pg1(ell=%g) = %g\n", ell, Pg1);


   /* Clean up */
   free(fmn);
   for (alpha=1; alpha<M_PAR; alpha++) free(dfmn_dp[alpha]);   


   return 0;
}


